/*-----------------------------------------------------*/
/*--	Micro-EMPIX Processes  (proc.c)               -*/
/*--    Ver 1.0 (c) 2004                              -*/
/*--	E             -*/
/*--    ==================================            -*/
/*--    .                  -*/
/*--    .                        -*/
/*--    ..                       -*/
/*-----------------------------------------------------*/

#ifndef _DEFS_H
	#define _DEFS_H
	#include "defs.h"
#endif

#ifndef _ASM_H
	#define _ASM_H
	#include "asm.h"
#endif


/* ====================================================*/
/* ============== Function Prototypes =================*/
/* ====================================================*/

int create (void *procaddr, int priority, char *name, int type);
int reschedule ();

/* ====================================================*/
/* ==========   ============*/
/* ====================================================*/

struct pcb proctab[MAXPROC];

int current; 					/*    */
int numproc; 					/*      .. */
int stacks[MAXPROC][STACKSIZE];  /*     */
int preempt; 					/*        */
int preregs[8]; 					/*      */

/*          .. */
/* void *procaddr:          */
/* int priority:     */
/* char *name:     */
/* int type:     */

int create (void *procaddr, int priority, char *name, int type)
{
	int ps;
	int i;

	disabl_intr(&ps); /*    */
	numproc++;		  /*        */
	proctab[numproc].pid=numproc; 					/*	 PID   */
	proctab[numproc].pstate=READY;					/*    */
	for (i=0;(i<PNMLEN-1) && (name[i]!='\0');i++)   /*     */
            proctab[numproc].pname[i]=name[i];
	proctab[numproc].pname[i]='\0';
	proctab[numproc].pprio=priority;				/*     */
	proctab[numproc].type=type;						/*     */

	for (i=0;i<14;i++)								/*    0 */
		proctab[numproc].pregs[i]=0;

    proctab[numproc].pregs[RIP]  = getoffset(enable_all_intr);  /* The offset of the function address  -> IP */
    proctab[numproc].pregs[RCS]  = getsegment(enable_all_intr); /* The segment of the function address -> CS */
	proctab[numproc].pregs[PSW] = INITPS;			   /*     */
	/* The offset of the stack (normally it would be -1 but we insert data inside */
	/* the stack in the following line) */
    proctab[numproc].pregs[RSP]  = getoffset (&stacks[numproc][STACKSIZE-2]);
    stacks[numproc][STACKSIZE-2]= getoffset(procaddr);
    stacks[numproc][STACKSIZE-1]= getsegment(procaddr);
    proctab[numproc].pregs[RSS]  = getsegment (stacks); /* The segment of the stack */
    proctab[numproc].pregs[RDS]  = _DS; /* The data segment is common for all processes */

	restor_intr (&ps); /*     */
	enable_all_intr(); /*    */

}

/*   reschedule    .     */
/*               */

int reschedule ()
{
	int previous;
	int ps;

	disabl_intr(&ps); /*    */
	preempt=QUANTUM;

	/*             */
	/* --------------------------------------------------------------------------- */

	if (current==-1) { /*         reschedule() */
	 previous=0; /*     PID=0 */
	 current=1;  /*       PID=1 */
 	}
	else { /*  */
	 previous=current; /*      */
	 current++; /*       */
	 if (current>numproc) current=1; /*          */
	 while (proctab[current].pstate==BLOCKED) { /*     blocked */
	  current++; /*    */
	  if (current>numproc) current=1; /*          */
	 }
	}
	/*----------------------------------------------------------------------------- */

	if (proctab[previous].pstate==RUNNING) proctab[previous].pstate=READY;
	proctab[current].pstate=RUNNING;

	/*   */
	ctxsw (proctab[previous].pregs,proctab[current].pregs);

	restor_intr (&ps); /*     */
	enable_all_intr(); /*    */

	return OK;
}

